// devices.box.viewmodel.ts
import { useAppDispatch, useAppSelector } from "@hooks";
import { RefObject, useCallback, useMemo, useState } from "react";
import { Device, setSelectedDevice } from "../reducer/device.reducer";
import { fetchPluginsThunk } from "../routines/plugin.routine";

export const useDeviceBoxViewModel = (searchRef: RefObject<HTMLInputElement>) => {
    const dispatch = useAppDispatch();
    const { devices, pnds, selected: selectedDevice } = useAppSelector(
        (state) => state.device
    );
    const [addModal, setAddModal] = useState(false);
    const [searchValue, setSearchValue] = useState(""); // Add search state

    const handleSearch = useCallback((value: string) => {
        setSearchValue(value);
    }, []);

    const handleItemClick = useCallback((device: Device) => {
        dispatch(setSelectedDevice({ device }));
    }, []);

    const openAddModal = useCallback(() => {
        dispatch(fetchPluginsThunk());
        setAddModal(true);
    }, []);

    const closeModal = useCallback(() => {
        setAddModal(false);
    }, []);

    const filteredDevices = useMemo(() => {
        if (!searchValue) return devices;

        return devices.filter((device) => {
            const user = pnds.find((pnd) => pnd.id === device.pid);
            return (
                device.id?.toLowerCase().includes(searchValue.toLowerCase()) ||
                device.name?.toLowerCase().includes(searchValue.toLowerCase()) ||
                user?.name?.toLowerCase().includes(searchValue.toLowerCase())
            );
        });
    }, [devices, pnds, searchValue]); // Now depends on searchValue instead of ref

    return {
        filteredDevices,
        handleItemClick,
        selectedDevice,
        pnds,
        addModal,
        openAddModal,
        closeModal,
        searchValue,
        handleSearch,
    };
};