import { faGripVertical, IconDefinition } from '@fortawesome/free-solid-svg-icons'
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import UpdateIndicator from '@layout/grid.layout/update-inidicator.layout/update-indicator.layout'
import { Category, CategoryType } from '@shared/types/category.type'
import { Col, Container, Row } from 'react-bootstrap'
import { Scrollbar } from '../scrollbar/Scrollbar.view'

interface GridBoxProps {
    title: string
    title_icon: IconDefinition
    children: React.ReactNode
    className?: string
    disabled?: boolean
}

export const GridBox: React.FC<GridBoxProps> = ({
    children,
    title,
    title_icon,
    className = '',
    disabled = false,
}) => {
    return (
        <div className="grid-box h-100">
            <Container
                fluid
                className={`c-box d-flex ${disabled && 'text-disabled disabled'} flex-column h-100 ${className}`}>
                <div>
                    {!disabled && (
                        <UpdateIndicator
                            category={Category.DEVICE as CategoryType}
                            updateInterval={15000}
                        />
                    )}
                    <FontAwesomeIcon icon={faGripVertical} className="drag-handle" />
                    <Row className="mb-0">
                        <Col xs={12}>
                            <h4 className={`c-box-title ${disabled && 'text-disabled'}`}>
                                <FontAwesomeIcon
                                    icon={title_icon}
                                    size="1x"
                                    className={`me-2 ${disabled ? 'text-disabled' : 'text-primary'}`}
                                />
                                {title}
                            </h4>
                        </Col>
                    </Row>
                </div>
                <Scrollbar scrollX={false} className="flex-grow-1 content">
                    {children}
                </Scrollbar>
            </Container>
        </div>
    )
}
